#include "keymap.h"
#include "layers.h"
#include "tapdance.h"


// Define custom keys
// (Must be done BEFORE keymaps)
enum custom_keycodes {
	RGB_SLD = EZ_SAFE_RANGE,
	HSV_136_138_245,
	HSV_24_210_255,
	HSV_0_0_255,
	ST_MACRO_0,
	ST_MACRO_1,
	RU_LAYER,
	EN_LAYER,
	RU_SYMBOL,
};

enum tap_dance_codes {
	TD_WMLAYOUT,
};


// Assemble key and LED maps
enum layer_indices {
	LAYER_MAIN_IDX,
	LAYER_COLEMAK_IDX,
	LAYER_NUMPAD_IDX,
	LAYER_SYMBOLS_IDX,
	LAYER_ARROWS_IDX,
	LAYER_FKEYS_IDX,
	LAYER_NAVIGATE_IDX,
	LAYER_DESKTOP_IDX,
	LAYER_KEYBOARD_IDX,
	LAYER_RUSSIAN_IDX,
	LAYER_RUSSIAN_SYMBOLS_IDX,
};
enum led_map_indices {
	LAYER_COLEMAK_LEDS_IDX,
	LAYER_NUMPAD_LEDS_IDX,
	LAYER_SYMBOLS_LEDS_IDX,
	LAYER_ARROWS_LEDS_IDX,
	LAYER_FKEYS_LEDS_IDX,
	LAYER_NAVIGATE_LEDS_IDX,
	LAYER_DESKTOP_LEDS_IDX,
	LAYER_RUSSIAN_LEDS_IDX,
	LAYER_RUSSIAN_SYMBOLS_LEDS_IDX,
};
const uint16_t PROGMEM keymaps[][MATRIX_ROWS][MATRIX_COLS] = {
	[LAYER_MAIN_IDX] = LAYER_MAIN,
	[LAYER_COLEMAK_IDX] = LAYER_COLEMAK,
	[LAYER_NUMPAD_IDX] = LAYER_NUMPAD,
	[LAYER_SYMBOLS_IDX] = LAYER_SYMBOLS,
	[LAYER_ARROWS_IDX] = LAYER_ARROWS,
	[LAYER_FKEYS_IDX] = LAYER_FKEYS,
	[LAYER_NAVIGATE_IDX] = LAYER_NAVIGATE,
	[LAYER_DESKTOP_IDX] = LAYER_DESKTOP,
	[LAYER_KEYBOARD_IDX] = LAYER_KEYBOARD,
	[LAYER_RUSSIAN_IDX] = LAYER_RUSSIAN,
	[LAYER_RUSSIAN_SYMBOLS_IDX] = LAYER_RUSSIAN_SYMBOLS,
};
const uint8_t PROGMEM ledmap[][DRIVER_LED_TOTAL][3] = {
	[LAYER_COLEMAK_LEDS_IDX] = LAYER_COLEMAK_LEDS,
	[LAYER_NUMPAD_LEDS_IDX] = LAYER_NUMPAD_LEDS,
	[LAYER_SYMBOLS_LEDS_IDX] = LAYER_SYMBOLS_LEDS,
	[LAYER_ARROWS_LEDS_IDX] = LAYER_ARROWS_LEDS,
	[LAYER_FKEYS_LEDS_IDX] = LAYER_FKEYS_LEDS,
	[LAYER_NAVIGATE_LEDS_IDX] = LAYER_NAVIGATE_LEDS,
	[LAYER_DESKTOP_LEDS_IDX] = LAYER_DESKTOP_LEDS,
	[LAYER_RUSSIAN_LEDS_IDX] = LAYER_RUSSIAN_LEDS,
	[LAYER_RUSSIAN_SYMBOLS_LEDS_IDX] = LAYER_RUSSIAN_SYMBOLS_LEDS
};


extern bool g_suspend_state;
extern rgb_config_t rgb_matrix_config;

void keyboard_post_init_user(void) {
	rgb_matrix_enable();
}



void set_layer_color(int layer) {
	for (int i = 0; i < DRIVER_LED_TOTAL; i++) {
		HSV hsv = {
			.h = pgm_read_byte(&ledmap[layer][i][0]),
			.s = pgm_read_byte(&ledmap[layer][i][1]),
			.v = pgm_read_byte(&ledmap[layer][i][2]),
		};
		if (!hsv.h && !hsv.s && !hsv.v) {
			rgb_matrix_set_color( i, 0, 0, 0 );
		} else {
			RGB rgb = hsv_to_rgb( hsv );
			float f = (float)rgb_matrix_config.hsv.v / UINT8_MAX;
			rgb_matrix_set_color( i, f * rgb.r, f * rgb.g, f * rgb.b );
		}
	}
}

void rgb_matrix_indicators_user(void) {
	if (g_suspend_state || keyboard_config.disable_layer_led) {
		return;
	}

	switch (biton32(layer_state)) {
		case 1:
			set_layer_color(1);
			break;
		case 2:
			set_layer_color(2);
			break;
		case 3:
			set_layer_color(3);
			break;
		case 4:
			set_layer_color(4);
			break;
		case 5:
			set_layer_color(5);
			break;
		case 6:
			set_layer_color(6);
			break;
		case 7:
			set_layer_color(7);
			break;
		case 9:
			set_layer_color(9);
			break;
		case 10:
			set_layer_color(10);
			break;
		default:
			if (rgb_matrix_get_flags() == LED_FLAG_NONE) {
				rgb_matrix_set_color_all(0, 0, 0);
			}
			break;
	}
}


bool process_record_user(uint16_t keycode, keyrecord_t *record) {
	switch (keycode) {
		case ST_MACRO_0:
			if (record->event.pressed) {
				SEND_STRING(SS_LCTL(SS_LGUI(SS_LSFT(SS_TAP(X_R)))));
			}
			break;

		case ST_MACRO_1:
			if (record->event.pressed) {
				SEND_STRING(SS_LGUI(SS_TAP(X_P)) SS_DELAY(100) SS_TAP(X_Y)  SS_DELAY(100) SS_TAP(X_ENTER));

			}
			break;

		case RU_LAYER:
			if (record->event.pressed) {
				SEND_STRING(SS_LGUI(SS_LSFT(SS_LCTL(SS_TAP(X_9)))));
				layer_move(9);
			}
			break;

		case EN_LAYER:
			if (record->event.pressed) {
				SEND_STRING(SS_LGUI(SS_LSFT(SS_LCTL(SS_TAP(X_0)))));
				layer_move(0);
			}
			break;

		case RU_SYMBOL:
			if (record->event.pressed) {
				SEND_STRING(SS_LGUI(SS_LSFT(SS_LCTL(SS_TAP(X_0)))));
				layer_move(10);
			} else {
				SEND_STRING(SS_LGUI(SS_LSFT(SS_LCTL(SS_TAP(X_9)))));
				layer_move(9);
			}
			break;

		case RGB_SLD:
			if (record->event.pressed) {
				rgblight_mode(1);
			}
			return false;

		case HSV_136_138_245:
			if (record->event.pressed) {
				rgblight_mode(1);
				rgblight_sethsv(136,138,245);
			}
			return false;

		case HSV_24_210_255:
			if (record->event.pressed) {
				rgblight_mode(1);
				rgblight_sethsv(24,210,255);
			}
			return false;

		case HSV_0_0_255:
			if (record->event.pressed) {
				rgblight_mode(1);
				rgblight_sethsv(0,0,255);
			}
			return false;
	}
	return true;
}

uint32_t layer_state_set_user(uint32_t state) {

	uint8_t layer = biton32(state);

	ergodox_board_led_off();
	ergodox_right_led_1_off();
	ergodox_right_led_2_off();
	ergodox_right_led_3_off();
	switch (layer) {
		case 1:
			ergodox_right_led_1_on();
			break;

		case 2:
			ergodox_right_led_2_on();
			break;

		case 3:
			ergodox_right_led_3_on();
			break;

		case 4:
			ergodox_right_led_1_on();
			ergodox_right_led_2_on();
			break;

		case 5:
			ergodox_right_led_1_on();
			ergodox_right_led_3_on();
			break;

		case 6:
			ergodox_right_led_2_on();
			ergodox_right_led_3_on();
			break;

		case 8:
			ergodox_right_led_1_on();
			ergodox_right_led_2_on();
			ergodox_right_led_3_on();
			break;

		default:
			break;
	}
	return state;
};


qk_tap_dance_action_t tap_dance_actions[] = {
	[TD_WMLAYOUT] = ACTION_TAP_DANCE_FN_ADVANCED(td_wmlayout_dance, td_wmlayout_finished, td_wmlayout_reset),
};
